@@ -9,8 +9,10 @@
 #' @param min.overlap the minimum number of overlaps. Only those terms with members that overlap with input data at least min.overlap (3 by default) will be processed
 #' @param test the test statistic used. It can be "fisher" for using fisher's exact test, "hypergeo" for using hypergeometric test, or "binomial" for using binomial test. Fisher's exact test is to test the independence between gene group (genes belonging to a group or not) and gene annotation (genes annotated by a term or not), and thus compare sampling to the left part of background (after sampling without replacement). Hypergeometric test is to sample at random (without replacement) from the background containing annotated and non-annotated genes, and thus compare sampling to background. Unlike hypergeometric test, binomial test is to sample at random (with replacement) from the background with the constant probability. In terms of the ease of finding the significance, they are in order: hypergeometric test > fisher's exact test > binomial test. In other words, in terms of the calculated p-value, hypergeometric test < fisher's exact test < binomial test
 #' @param background.annotatable.only logical to indicate whether the background is further restricted to annotatable genes (covered by 'annotation.file'). In other words, if the background is provided, the background genes are those after being overlapped with annotatable genes. Notably, if only one annotation is provided in 'annotation.file', it should be false
+#' @param p.tail the tail used to calculate p-values. It can be either "two-tails" for the significance based on two-tails (ie both over- and under-overrepresentation)  or "one-tail" (by default) for the significance based on one tail (ie only over-representation)
 #' @param p.adjust.method the method used to adjust p-values. It can be one of "BH", "BY", "bonferroni", "holm", "hochberg" and "hommel". The first two methods "BH" (widely used) and "BY" control the false discovery rate (FDR: the expected proportion of false discoveries amongst the rejected hypotheses); the last four methods "bonferroni", "holm", "hochberg" and "hommel" are designed to give strong control of the family-wise error rate (FWER). Notes: FDR is a less stringent condition than FWER
 #' @param verbose logical to indicate whether the messages will be displayed in the screen. By default, it sets to false for no display
+#' @param silent logical to indicate whether the messages will be silent completely. By default, it sets to false. If true, verbose will be forced to be false
 #' @return 
 #' an object of class "eTerm", a list with following components:
 #' \itemize{
@@ -24,6 +26,9 @@
 #'  \item{\code{zscore}: a vector containing z-scores}
 #'  \item{\code{pvalue}: a vector containing p-values}
 #'  \item{\code{adjp}: a vector containing adjusted p-values. It is the p value but after being adjusted for multiple comparisons}
+#'  \item{\code{or}: a vector containing odds ratio}
+#'  \item{\code{CIl}: a vector containing lower bound confidence interval for the odds ratio}
+#'  \item{\code{CIu}: a vector containing upper bound confidence interval for the odds ratio}
 #'  \item{\code{cross}: a matrix of nTerm X nTerm, with an on-diagnal cell for the overlapped-members observed in an individaul term, and off-diagnal cell for the overlapped-members shared betwene two terms}
 #'  \item{\code{call}: the call that produced this result}
 #' }
@@ -63,7 +68,8 @@
 #' 
 #' # Using ImmunoBase SNPs and associations/annotations with disease traits
 #' ## get ImmunoBase
-#' ImmunoBase <- xRDataLoader(RData.customised='ImmunoBase')
+#' RData.location <- "http://galahad.well.ox.ac.uk/bigdata_dev/"
+#' ImmunoBase <- xRDataLoader(RData.customised='ImmunoBase', RData.location=RData.location)
 #' ## get disease associated variants/SNPs
 #' variants_list <- lapply(ImmunoBase, function(x) cbind(SNP=names(x$variants), Disease=rep(x$disease,length(x$variants))))
 #' ## extract annotations as a data frame: Variant Disease_Name 
@@ -71,7 +77,7 @@
 #' head(annotation.file)
 #' ## provide the input SNPs of interest
 #' ## for example, cis-eQTLs induced by interferon gamma
-#' cis <- xRDataLoader(RData.customised='JKscience_TS2A')
+#' cis <- xRDataLoader(RData.customised='JKscience_TS2A', RData.location=RData.location)
 #' data.file <- matrix(cis[which(cis$IFN_t>0),c('variant')], ncol=1)
 #' # perform enrichment analysis
 #' eTerm <- xEnricherYours(data.file=data.file, annotation.file=annotation.file)
@@ -82,23 +88,29 @@
 #' print(bp)
 #' }
 
-xEnricherYours <- function(data.file, annotation.file, background.file=NULL, size.range=c(10,2000), min.overlap=3, test=c("hypergeo","fisher","binomial"), background.annotatable.only=T, p.adjust.method=c("BH", "BY", "bonferroni", "holm", "hochberg", "hommel"), verbose=T)
+xEnricherYours <- function(data.file, annotation.file, background.file=NULL, size.range=c(10,2000), min.overlap=3, test=c("hypergeo","fisher","binomial"), background.annotatable.only=T, p.tail=c("one-tail","two-tails"), p.adjust.method=c("BH", "BY", "bonferroni", "holm", "hochberg", "hommel"), verbose=T, silent=FALSE)
 {
     startT <- Sys.time()
-    message(paste(c("Start at ",as.character(startT)), collapse=""), appendLF=T)
-    message("", appendLF=T)
+    if(!silent){
+    	message(paste(c("Start at ",as.character(startT)), collapse=""), appendLF=TRUE)
+    	message("", appendLF=TRUE)
+    }else{
+    	verbose <- FALSE
+    }
+    
     ####################################################################################
     
     ## match.arg matches arg against a table of candidate values as specified by choices, where NULL means to take the first one
     test <- match.arg(test)
     p.adjust.method <- match.arg(p.adjust.method)
+    p.tail <- match.arg(p.tail)
     
     ############
     if(length(data.file)==0){
-    	return(FALSE)
+    	return(NULL)
     }
     ############
-        
+    
     ###################
     ## import data file
     if(is.matrix(data.file) | is.data.frame(data.file)){
@@ -134,7 +146,7 @@ xEnricherYours <- function(data.file, annotation.file, background.file=NULL, siz
 		}
     }else{
     	background <- background.file
-    }    
+    }
     
     ## import annotation file
     if(is.matrix(annotation.file) | is.data.frame(annotation.file)){
@@ -157,10 +169,10 @@ xEnricherYours <- function(data.file, annotation.file, background.file=NULL, siz
     ## define annotation information
 	anno <- split(x=input[,1], f=input[,2])
     
-	## define ontology information (artifically)
+	## define ontology information (artificially)
 	terms <- names(anno)
-	nodes <- data.frame(name=terms, term_id=terms, term_name=terms, term_distance=rep(1,length(terms)), stringsAsFactors=F)
-	root <- c('Root', 'Root', 'Root', 0)
+	nodes <- data.frame(name=terms, term_id=terms, term_name=terms, term_distance=rep(1,length(terms)), term_namespace=rep('Customised',length(terms)), stringsAsFactors=F)
+	root <- c('Root', 'Root', 'Root', 0, 'Customised')
 	nodes <- rbind(nodes, root)
 	relations <- data.frame(from='Root', to=nodes$name)
 	g <- igraph::graph.data.frame(d=relations, directed=T, vertices=nodes)
@@ -174,7 +186,7 @@ xEnricherYours <- function(data.file, annotation.file, background.file=NULL, siz
         message(sprintf("'xEnricher' is being called (%s):", as.character(now)), appendLF=T)
         message(sprintf("#######################################################", appendLF=T))
     }
-    eTerm <- xEnricher(data=data, annotation=anno, g=g, background=background, size.range=size.range, min.overlap=min.overlap, test=test, p.adjust.method=p.adjust.method, ontology.algorithm="none",true.path.rule=F, verbose=verbose)
+    eTerm <- xEnricher(data=data, annotation=anno, g=g, background=background, size.range=size.range, min.overlap=min.overlap, test=test, p.tail=p.tail, p.adjust.method=p.adjust.method, ontology.algorithm="none",true.path.rule=F, verbose=verbose)
 	
 	if(verbose){
         now <- Sys.time()
@@ -185,10 +197,12 @@ xEnricherYours <- function(data.file, annotation.file, background.file=NULL, siz
     
     ####################################################################################
     endT <- Sys.time()
-    message(paste(c("\nEnd at ",as.character(endT)), collapse=""), appendLF=T)
-    
     runTime <- as.numeric(difftime(strptime(endT, "%Y-%m-%d %H:%M:%S"), strptime(startT, "%Y-%m-%d %H:%M:%S"), units="secs"))
-    message(paste(c("Runtime in total is: ",runTime," secs\n"), collapse=""), appendLF=T)
+    
+    if(!silent){
+    	message(paste(c("\nEnd at ",as.character(endT)), collapse=""), appendLF=TRUE)
+    	message(paste(c("Runtime in total is: ",runTime," secs\n"), collapse=""), appendLF=TRUE)
+    }
     
     invisible(eTerm)
 }
